#include <Arduino.h>
#include <RTClib.h>
#include <SPI.h>
#include <SD.h>

#define SDCARD_CS_PIN 4
// valves are normally closed
#define ONSHORE_PIN PIN_A0
#define OFFSHORE_PIN PIN_A1
// how frequently to update
#define DECISION_PERIOD_MS 30000

RTC_PCF8523 rtc;
char entry_buffer[512];
File logfile;
DateTime now;
volatile float wind_direction;
volatile float wind_velocity;

enum decision
{
  ONSHORE = 0,
  OFFSHORE = 1,
  UNCERTAIN = 2
};

void setup()
{
  /*
   *  Real Time Clock
   */
  rtc.begin();
  if (!rtc.initialized() || rtc.lostPower())
  {
    Serial.println("RTC is not initialized or lost power, setting time...");
    // When time needs to be set on a new device, or after a power loss, the
    // following line sets the RTC to the date & time this sketch was compiled
    DateTime set_time = DateTime(F(__DATE__), F(__TIME__));
    rtc.adjust(set_time);
    Serial.write(sprintf("RTC datetime set to %s", set_time.toString("MMM-DD-YYYY hh:mm:ss")));
  }
  else
  {
    now = rtc.now();
    Serial.write(sprintf("RTC time: %s", now.toString("MMM-DD-YYYY hh:mm:ss")));
  }

  // make sure stop bit is cleared and rtc is running
  rtc.start();

  /*
   *  MicroSD card
   */
  // setup sd card pins
  SD.begin(SDCARD_CS_PIN);
  // figure out filename using newly set datetime
  DateTime now = rtc.now();
  char filename_string[255];
  sprintf(filename_string, "%s.txt", now.toString("YYYY-MM-DD"));
  // open logfile
  logfile = SD.open(filename_string);

  /*
   *  Met Station
   */
  // setup met station serial

  /*
   * Digital valve control pins
   */
  pinMode(ONSHORE_PIN, OUTPUT);
  pinMode(OFFSHORE_PIN, OUTPUT);
}

void loop()
{
  // grab reading

  // grab time
  now = rtc.now();

  // decide on sampler settings
  decision d = decide(wind_direction, wind_velocity);

  // assemble log entry and write to SD card
  // csv format
  // DATETIME,WIND_DIRECTION,WIND_VELOCITY,DECISION
  sprintf(entry_buffer, "%s,%f.2,%f.2,%i\n", now.toString("YYYY-MM-DD hh:mm:ss"), wind_direction, wind_velocity, d);
  logfile.write(entry_buffer);
  logfile.flush();

  // delay until next decision time
  delay(DECISION_PERIOD_MS);
}

decision decide(float direction, float velocity)
{
  return UNCERTAIN;
}

void sample_onshore()
{
  // open sampling to onshore valve
  digitalWrite(ONSHORE_PIN, HIGH);
  // wait a moment
  delay(500);
  // close sampling for offshore valve
  digitalWrite(OFFSHORE_PIN, LOW);
}

void sample_offshore()
{
  // open sampling to offshore valve
  digitalWrite(OFFSHORE_PIN, HIGH);
  // wait a moment
  delay(500);
  // close sampling for onshore valve
  digitalWrite(ONSHORE_PIN, LOW);
}

void sample_both()
{
  digitalWrite(ONSHORE, HIGH);
  digitalWrite(OFFSHORE, HIGH);
}